
# q-expansion of newform 3234.2.a.v, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 3234
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [1], [4], [0], [-1], [6], [4], [2], [-8], [-6], [-6], [-6], [-12], [4], [-6], [2], [0], [-10], [4], [-12], [0], [-16], [14], [14], [14], [-14], [6], [-8], [6], [14], [0], [2], [14], [-2], [-10], [-8], [4], [4], [12], [-2], [-4], [8], [0], [-14], [10], [-2], [-4], [-6], [-6], [-8], [-14], [-16], [8], [16], [-26], [0], [4], [4], [-10], [26], [14], [14], [14], [-10], [34], [-6], [20], [6], [8], [-6], [6], [24], [34], [-14], [20], [22], [6], [20], [-26], [-4], [-12], [38], [-24], [-26], [20], [28], [14], [10], [-14], [-24], [-36], [-24], [20], [-28], [-28], [-20], [-20], [-18], [22], [-14], [8], [-30], [26], [10], [-32], [34], [-24], [12], [12], [-4], [8], [-6], [26], [4], [28], [18], [-32], [-26], [30], [-24], [0], [-10], [-34], [-36], [-24], [-18], [-30], [-18], [42], [-50], [-16], [16], [-28], [2], [-24], [-28], [24], [50], [-24], [-2], [-26], [-26], [44], [-28], [-16], [54], [6], [40], [-40], [-16], [-18], [50], [28], [24], [52], [24], [32], [-18], [32], [30], [36], [30], [-32], [36], [2], [42], [8], [26], [58], [-44], [32], [4], [0], [-10], [30], [-46], [-40], [6], [12], [34], [-8], [-30], [18], [-24], [-36], [-20], [26], [60], [6], [-4], [-38], [48], [12], [24], [-52], [-50], [30], [26], [12], [36], [-50], [-4], [20], [0], [60], [28], [70], [-44], [-66], [-68], [30], [-48], [16], [-48], [-60], [10], [-8], [8], [28], [22], [58], [-4], [-50], [-56], [34], [54], [-22], [12], [-28], [-48], [-22], [-70], [-16], [70], [-44], [-36], [-50], [6], [-36], [26], [-60], [10], [-10], [-34], [64], [28], [30], [32], [8], [-64], [-38], [60], [52], [-42], [-50], [-48], [74], [38], [12], [-24], [-8], [36], [30], [18], [42], [-56], [-62], [42], [-24], [26], [76], [12], [-46], [16], [-40], [-38], [-20], [-46], [34], [46], [24], [-56], [-24], [-38], [66], [0], [-26], [-18], [30], [32], [50], [-12], [-54], [54], [46], [10], [38], [-20], [-54], [70], [52], [-50], [-32], [12], [4], [78], [36], [-8], [36], [38], [0], [30], [-54], [-60], [-32], [56], [16], [8], [-54], [18], [-6], [-38], [-58], [76], [-74], [34], [-60], [-20], [24], [70], [42], [-18], [28], [34], [24], [36], [-78], [64], [-66], [8], [46], [-6], [14], [-68], [-18], [-92], [-16], [44], [16], [-64], [22], [-10], [18], [-24], [6], [-16], [-14], [34], [-18], [52], [86], [-16], [52], [68], [-20], [2], [-74], [-96], [-22], [36], [36], [-72], [-6], [-72], [62], [76], [-80], [28], [44], [-56], [40], [18], [32], [-58], [70], [6], [-44], [32], [36], [-54], [-38], [-48], [4], [-48], [44], [38], [-24], [-70], [-12], [-42], [-58], [-78], [88], [98], [38], [28], [86], [78], [-76], [68], [16], [34], [12], [-48], [-50], [-80], [32], [26], [10], [24], [30], [78], [68], [54], [10], [24], [-36], [50], [-100], [24]]
